'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

exports.default = function (_ref) {
	var t = _ref.types;

	function createType(type) {
		if (type.typeParameters === null) {
			return type.id;
		} else if (type.type === "AnyTypeAnnotation") {
			return t.nullLiteral();
		} else {
			var params = type.typeParameters.params.map(createType);
			return createObjectExpression({
				type: type.id,
				params: params
			});
		}
	}

	function processComments() {
		var comments = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

		return comments.filter(function (comment) {
			return comment.type === 'CommentBlock' && comment.value.startsWith('*');
		}).map(function (comment) {
			return comment.value.slice(1).trim();
		});
	}

	function createObjectExpression(obj) {
		var entries = [];

		for (var prop in obj) {
			if (obj.hasOwnProperty(prop)) {
				var value = createExpression(obj[prop]);
				entries.push(t.objectProperty(t.identifier(prop), value));
			}
		}

		return t.ObjectExpression(entries);
	}

	function createDecoratorList() {
		var decorators = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

		return decorators.map(function (_ref2) {
			var expression = _ref2.expression;
			return expression;
		});
	}

	function createExpression(value) {
		if ((0, _babelTypes.isNode)(value)) {
			return value;
		}

		if (value instanceof Array) {
			return t.arrayExpression(value.map(createExpression));
		}

		if (value === null) {
			return t.nullLiteral();
		}

		switch (typeof value === 'undefined' ? 'undefined' : _typeof(value)) {
			case 'string':
				return t.stringLiteral(value);
			case 'boolean':
				return t.booleanLiteral(value);
			case 'object':
				return createObjectExpression(value);
		}

		return t.nullLiteral();
	}

	var visitor = {
		ClassProperty: function ClassProperty(path) {
			var _ref3;

			var node = path.node;
			var type = node.typeAnnotation;

			var decorators = node.decorators || [];
			var leadingComments = node.leadingComments || [];

			// Make sure we get all the comments on class properties, as these behave weirdly
			// when combined with decorators
			var comments = [].concat(_toConsumableArray((_ref3 = []).concat.apply(_ref3, _toConsumableArray(decorators.map(function (_ref4) {
				var _ref4$leadingComments = _ref4.leadingComments;
				var leadingComments = _ref4$leadingComments === undefined ? [] : _ref4$leadingComments;
				return leadingComments;
			})))), _toConsumableArray(leadingComments));

			var filteredComments = [];

			// Sometimes duplicate comments appear, so remove duplicates based on node position
			comments.forEach(function (comment) {
				var contains = false;
				var _iteratorNormalCompletion = true;
				var _didIteratorError = false;
				var _iteratorError = undefined;

				try {
					for (var _iterator = filteredComments[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
						var item = _step.value;

						if (item.start === comment.start && item.end === comment.end) {
							contains = true;
						}
					}
				} catch (err) {
					_didIteratorError = true;
					_iteratorError = err;
				} finally {
					try {
						if (!_iteratorNormalCompletion && _iterator.return) {
							_iterator.return();
						}
					} finally {
						if (_didIteratorError) {
							throw _iteratorError;
						}
					}
				}

				if (!contains) {
					filteredComments.push(comment);
				}
			});

			if (type) {
				this.definition.properties.push({
					key: node.key.name,
					initializer: node.value === null ? null : t.arrowFunctionExpression([], node.value),
					decorators: createDecoratorList(node.decorators),
					docstrings: processComments(filteredComments),
					type: createType(type.typeAnnotation),
					static: node.static
				});
			}
		},
		ClassMethod: function ClassMethod(path) {
			var node = path.node;
			var id = node.key;
			var params = node.params;
			var body = node.body;
			var generator = node.generator;
			var isAsync = node.async;
			var kind = node.kind;
			var returnType = node.returnType;

			var replaceSupers = new _babelHelperReplaceSupers2.default({
				forceSuperMemoisation: false,
				methodPath: path,
				methodNode: node,
				objectRef: node.static ? t.thisExpression() : t.memberExpression(t.thisExpression(), t.identifier('constructor')),
				isStatic: true,
				isLoose: false,
				file: this.file
			}, true);

			replaceSupers.replace();

			var methodDefintion = {
				key: node.key.name,
				static: node.static,
				decorators: createDecoratorList(node.decorators || []),
				docstrings: processComments(node.leadingComments),
				generator: generator,
				async: !!isAsync,
				function: t.functionExpression(id, params, body, generator, isAsync),
				params: params.map(function (_ref5) {
					var name = _ref5.name;
					var type = _ref5.typeAnnotation;

					return {
						name: name,
						type: type ? createType(type.typeAnnotation) : null
					};
				}),
				returnType: returnType ? createType(returnType.typeAnnotation) : null
			};

			switch (kind) {
				case 'get':
					this.definition.getters.push(methodDefintion);
					break;
				case 'set':
					this.definition.setters.push(methodDefintion);
					break;
				case 'method':
					this.definition.methods.push(methodDefintion);
					break;
				case 'constructor':
					var _iteratorNormalCompletion2 = true;
					var _didIteratorError2 = false;
					var _iteratorError2 = undefined;

					try {
						for (var _iterator2 = replaceSupers.bareSupers[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
							var bareSuper = _step2.value;

							bareSuper.replaceWith((0, _babelHelperOptimiseCallExpression2.default)(t.callExpression(t.memberExpression(t.identifier("Object"), t.identifier("getPrototypeOf")), [t.memberExpression(t.thisExpression(), t.identifier('constructor'))]), t.thisExpression(), bareSuper.node.arguments));
						}
					} catch (err) {
						_didIteratorError2 = true;
						_iteratorError2 = err;
					} finally {
						try {
							if (!_iteratorNormalCompletion2 && _iterator2.return) {
								_iterator2.return();
							}
						} finally {
							if (_didIteratorError2) {
								throw _iteratorError2;
							}
						}
					}

					var applyCall = function applyCall(fn) {
						return t.callExpression(t.memberExpression(fn, t.identifier("apply")), [t.thisExpression(), t.identifier('arguments')]);
					};

					this.definition.constructor = t.arrowFunctionExpression([], t.functionExpression(t.identifier('constructor'), [], path.get('body').node));
					break;
				default:
					throw new Error('Unsupported method type: ' + kind);
			}
		}
	};

	return {
		visitor: {
			Program: function Program(path, _ref6) {
				var _this = this;

				var opts = _ref6.opts;
				var file = _ref6.file;
				var filename = file.opts.filename;

				this.skip = false;

				var globs = opts.globs instanceof Array ? opts.globs : [opts.globs];

				globs.forEach(function (glob) {
					if (glob && !(0, _minimatch2.default)(filename, glob)) {
						// path.skip() breaks other transforms so don't use it
						_this.skip = true;
					}
				});
			},
			ClassDeclaration: function ClassDeclaration(path, _ref7) {
				var opts = _ref7.opts;
				var file = _ref7.file;

				// Glob did not match this file
				if (this.skip) {
					return;
				}

				var node = path.node;
				var id = node.id;
				var decorators = node.decorators;
				var superClass = node.superClass;

				if (superClass === null) {
					return;
				}

				var definition = {
					properties: [],
					setters: [],
					getters: [],
					methods: [],
					name: id.name,
					superClass: superClass,
					docstrings: processComments(node.leadingComments),
					decorators: createDecoratorList(decorators || []),
					implements: (node.implements || []).map(function (_ref8) {
						var id = _ref8.id;
						return id;
					})
				};

				path.traverse(visitor, { definition: definition, file: this.file });

				var options = createExpression(definition);

				path.replaceWith(t.variableDeclaration('var', [t.variableDeclarator(t.identifier(definition.name), t.callExpression(t.memberExpression(definition.superClass, t.identifier('define')), [options]))]));
			},
			ClassExpression: function ClassExpression(path, _ref9) {
				var opts = _ref9.opts;
				var file = _ref9.file;

				// Glob did not match this file
				if (this.skip) {
					return;
				}

				var node = path.node;
				var id = node.id;
				var decorators = node.decorators;
				var superClass = node.superClass;

				if (superClass === null) {
					return;
				}

				var name = id ? id.name : null;

				var definition = {
					properties: [],
					setters: [],
					getters: [],
					methods: [],
					name: name,
					superClass: superClass,
					docstrings: processComments(node.leadingComments),
					decorators: createDecoratorList(decorators || []),
					implements: (node.implements || []).map(function (_ref10) {
						var id = _ref10.id;
						return id;
					})
				};

				path.traverse(visitor, { definition: definition, file: this.file });

				var options = createExpression(definition);

				path.replaceWith(t.callExpression(t.memberExpression(definition.superClass, t.identifier('define')), [options]));
			}
		}
	};
};

var _babelTypes = require('babel-types');

var _minimatch = require('minimatch');

var _minimatch2 = _interopRequireDefault(_minimatch);

var _babelHelperReplaceSupers = require('babel-helper-replace-supers');

var _babelHelperReplaceSupers2 = _interopRequireDefault(_babelHelperReplaceSupers);

var _babelHelperOptimiseCallExpression = require('babel-helper-optimise-call-expression');

var _babelHelperOptimiseCallExpression2 = _interopRequireDefault(_babelHelperOptimiseCallExpression);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }